package App::EventStreamr::DVswitch::Sync;
use Method::Signatures;
use Moo;
use namespace::clean;

# ABSTRACT: Sync

our $VERSION = '0.5'; # VERSION: Generated by DZP::OurPkg:Version


extends 'App::EventStreamr::Sync';

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

App::EventStreamr::DVswitch::Sync - Sync

=head1 VERSION

version 0.5

=head1 SYNOPSIS

Manage Sync process.

=head1 DESCRIPTION

Manage all the configured devices designated to be 
running as ingest roles.

=head1 AUTHOR

Leon Wright < techman@cpan.org >

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2014 by Leon Wright.

This is free software, licensed under:

  The GNU Affero General Public License, Version 3, November 2007

=cut
