
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006009
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
EventStreamr
eventstreamr
cmds
cmd
api
Nicholls
extendable
TODO
devmon
cli
alsa
mgr
utilise
Orchestrator
DVswitch
IceCast
YouTube
Leon
Wright
techman
lib
App
Config
Ingest
ALSA
DV
IngestTest
V4L
Mixer
Record
Roles
MixerWait
Standby
Stream
Sync
Youtube
Devices
Internal
API
Devmon
Process
ConfigAPI
Logger
ProcessControl
Status
Utils
